static char rcs_id[] = "$Header: /home/kig/work/convert/src/RCS/codes.c,v 1.1 1996/10/20 19:37:31 kig Exp kig $";
/*******************************************************************************
 *
 * FILENAME:	codes.c
 * MODULE:		convert
 *
 * Copyright (c) 1996 Konstantin Gredeskoul
 * All rights reserved
 *
 * Internet:	kig@cyber.com.au
 * Post:		P.O.Box 500, Carnegie, VIC 3163, Australia
 *
 ******************************************************************************/

/* includes */
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "debuglog.h"
#include "main.h"


void log_codes(CODE_DATA *codes)
{
/*******************************************************************************
 *
 * PURPOSE
 *	This function prints to a log file all codes from codes structure
 *
 ******************************************************************************/
	int		cnt = 0;
	int		LL = L_TRC;
	char	func[] = "log_codes";

	debuglog((LL, func, "total of <%d> relations", codes->len));
	debuglog((LL, func, "left   right"));
	debuglog((LL, func, "------------"));

	for (cnt = 0; cnt < codes->len; cnt++) 
		debuglog((
			LL, 
			func, 
			" %c <---> %c", 
			codes->left[cnt], 
			codes->right[cnt]
			));

	debuglog((LL, func, "------------"));

	return;
}
 


void load_codes(FILE *fp_codefile, CODE_DATA *codes)	
{
/*******************************************************************************
 *
 * PURPOSE
 *	This function loads codes from a stream pointed to by fp_codefile and fills
 *	*codes with these values.
 *
 ******************************************************************************/

	int			rv = 0;
	char		buf[LINE_SIZE + 1];
	char		func[] = "load_codes";
	int			line_cnt = 0;

	short		s_left;
	short		s_right;

	int			i_left;
	int			i_right;

	char		c_left;
	char		c_right;

	int			read_mode = MODE_CHAR;
	int			i = 0;


	debuglog((L_DTL, func, "entering"));

	codes->len = 0;

	/*
	 * start reading file
	 */
	while ((rv = fgets(buf, LINE_SIZE, fp_codefile) != NULL))
	{
		/* increment line count */
		line_cnt++;

		/* check size of codes and compare with max size */
		if (codes->len >= CODE_ARRAY_SIZE)
		{
			debuglog((L_WRN, func, 
				"warning: maximum count of relations %d reached, skipping",
				CODE_ARRAY_SIZE));
			fprintf(stderr,
				"warning: maximum count of relations reached, skipping\n.");
			break;
		}

		/* handle commands such as /ASC and /CHR */
		if (buf[0] == CHAR_CMD)
		{
			debuglog((L_TRC, func, "command detected: <%.*s>", 
				CMD_LEN, buf));

			for (i = 0; i < strlen(buf); i++)
				*(buf + i) = toupper(*(buf + i));

			if (strncmp(buf, CMD_ASCII_CODES, CMD_LEN) == 0)
			{
				read_mode = MODE_ASCII_CODES;
				debuglog((L_INF, func, "mode set to ASC, line %d",
					line_cnt));
			}
			else if (strncmp(buf, CMD_HEX_CODES, CMD_LEN) == 0)
			{
				read_mode = MODE_HEX_CODES;
				debuglog((L_INF, func, "mode set to HEX, line %d",
					line_cnt));
			}
			else if (strncmp(buf, CMD_CHAR, CMD_LEN) == 0)
			{
				read_mode = MODE_CHAR;
				debuglog((L_INF, func, "mode set to CHR, line %d",
					line_cnt));
			}
			else
			{
				for (i = 0; i < strlen(buf); i++)
					if (*(buf + i) == '\n' || *(buf + i) == '\r')
					{
						*(buf + i) = '\0';
						break;
					}

				debuglog((L_WRN, func, 
					"unrecognized command token in line %d: <%s>", 
					line_cnt, buf));
			}
		}

		/* handle data lines (if not # or a blank line) */
		else if (buf[0] != CHAR_COMMENT && buf[0] != '\n')
		{
			/* read ASCII codes if appropriate */
			if (read_mode == MODE_ASCII_CODES)
				if (sscanf(buf, "%d %d", &i_left, &i_right) == EOF)
				{
					debuglog((L_ERR, func, 
						"corrupted input from code file, line no. <%d>", 
						line_cnt));
					debuglog((L_ERR, func,
						"trouble line is <%s>", buf));
					fprintf(stderr, 
						"warning: corrupted line %d in the code file",
						line_cnt);
					continue;
				}
				else
				{
					if (i_left < 256 && i_left > -1  &&
						i_right < 256 && i_right > -1)
					{
						c_left = (char)i_left;
						c_right = (char)i_right;
					}
					else
					{
						fprintf(stderr, 
							"warning: ASCII values in line %d"
							"of the code file are out of bounds - ignored\n", 
							line_cnt);
						debuglog((L_ERR, func, 
							"ASCII values in line %d are out of bounds",
							line_cnt));
					}
				}

			/* read characters instead */
			else if (read_mode == MODE_CHAR)
			{
				if (sscanf(buf, "%c %c", &c_left, &c_right) == EOF)
				{
					debuglog((L_ERR, func, 
						"corrupted input from code file, line no. <%d>", 
						line_cnt));
					debuglog((L_ERR, func,
						"trouble line is <%s>", buf));
					fprintf(stderr, 
						"warning: corrupted line %d in the code file",
						line_cnt);
					continue;
				}
			}
			/* hex numbers */
			else if (read_mode == MODE_HEX_CODES)
				if (sscanf(buf, "%hx %hx", &s_left, &s_right) == EOF)
				{
					debuglog((L_ERR, func, 
						"corrupted input from code file, line no. <%d>", 
						line_cnt));
					debuglog((L_ERR, func,
						"trouble line is <%s>", buf));
					fprintf(stderr, 
						"warning: corrupted line %d in the code file",
						line_cnt);
					continue;
				}
				else
				{
					if (s_left < 256 && s_left > -1  &&
						s_right < 256 && s_right > -1)
					{
						c_left = (char)s_left;
						c_right = (char)s_right;
					}
					else
					{
						fprintf(stderr, 
							"warning: HEX ASCII values in line %d"
							"of the code file are out of bounds - ignored\n", 
							line_cnt);
						debuglog((L_ERR, func, 
							"HEX ASCII values in line %d are out of bounds",
							line_cnt));
					}
				}
			/* this shoudln't normally happen */
			else
			{
				fprintf(stderr, 
					"error: unexpected read_mode = <%d>!\n",
					read_mode);
				debuglog((L_ERR, func, "unexpected read_mode = <%d>",
					read_mode));
			}
	
			/* fill codes with data read */
			*(codes->left + codes->len) = c_left;
			*(codes->right + codes->len) = c_right;

			codes->len++;
		}
	}

	/* print codes to a log file */
	log_codes(codes);

	debuglog((L_DTL, func, "leaving"));

	return;
}


