static char rcs_id[] = "$Header: /home/kig/work/convert/src/RCS/cyrfrom.c,v 1.1 1996/10/20 19:37:31 kig Exp kig $";
/*******************************************************************************
 *
 * FILENAME:	cyrfrom.c
 * MODULE:		convert
 * NOTES:		Cyrillic transliteration routines (FROM)
 *				translit_from_cyr() function
 *
 * Copyright (c) 1996 Konstantin Gredeskoul
 * All rights reserved
 *
 * Internet:	kig@cyber.com.au
 * Post:		P.O.Box 500, Carnegie, VIC 3163, Australia
 *
 ******************************************************************************/


/* includes */
#include <stdio.h>
#include <sys/stat.h>

#include "debuglog.h"
#include "bool.h"
#include "main.h"
#include "cyrfrom.h"


int translit_from_cyr(
	FILE *fp_infile, 			/* input file pointer */ 
	FILE *fp_outfile, 			/* output file pointer */
	char sepchar				/* exclude character */
	)
{
/*******************************************************************************
 *
 * PURPOSE
 *	This function converts text in Russian (Alternative encoding for MS-DOS)
 *	to English-transliterated text, eg Russian text typed in using English 
 *	characters. The conversion rules are in cyrfrom.h file.
 *
 ******************************************************************************/

	int			cin = 0;
	char		*pcout;

	/* boolean flags used during conversion */
	int			bool_exclude = FALSE;
	int			bool_convert = TRUE;
	int			bool_wrote = FALSE;

	int			array_size;

	/* fast looping variables */
	long		cnt = 0;
	int			i = 0;
	int			j = 0;
	long		infile_size = -1;

	char		func[] = "translit_from_cyr";

	int			perc_curr = -1;
	int			perc_prev = -1;

	int			fd;
	struct stat	stat_buff;


	debuglog((L_DTL, func, "entering"));



	/* get file size for status info */

	fd = fileno(fp_infile);
	if (fstat(fd, &stat_buff) != 0)
		debuglog((L_WRN, func, "couldn't get file stat for input file"));
	else
	{
		infile_size = stat_buff.st_size;
		debuglog((L_INF, func, "got input file size = %dKb",
			infile_size / KILOBYTE ));
	}


	/* check if exclude character is used */
	if (sepchar != '\0')
		bool_exclude = TRUE;


	/* 
	 * Set the starting pointer, 
	 * where pcode->left is Russian "A"
	 * and  pcode->right is English "A"
	 */
	pcode = (cyrcode *)codes;



	/* get and log array size */
	array_size = sizeof(codes) / sizeof(codes[0]);
	debuglog((L_INF, func, "array size for the alphabet is %d", array_size));



	/* log code table to a log file */
	for (i = 0; i < array_size; i++)
		debuglog((L_INF, func, " %s ----> %s",
			(pcode + i)->left, (pcode + i)->right));


	debuglog((L_DTL, func, "starting conversion"));

	/* start loop */
	while ((cin = getc(fp_infile)) != EOF)
	{
		cnt++;

		/* update status display */
		if (cnt % PERCENT_BLOCK == 0)
		{
			perc_curr = (int)(100.0 * (float)cnt/(float)infile_size);
			if (perc_curr > perc_prev)
			{
				fprintf(stderr, "status: %d%% complete...\r", perc_curr);
				perc_prev = perc_curr;
			}
		}

		/* if there was an exclude character, handle
		 * input correspondingly
		 */
		if (bool_exclude)
		{
			if (cin == sepchar)
			{
				bool_convert = !bool_convert;

				debuglog((L_INF, func,
					"found on/off char <%c> at byte <%d>", sepchar, cnt));

				if (bool_convert)
					debuglog((L_INF, func, "converting is ON"));
				else
					debuglog((L_INF, func, "converting is OFF"));
			}
		}

		/* if conversion is ON, then match the input
		 * character with the left hand side of the array
		 * and if matched - convert it */

		if (bool_convert)
			for (i = 0; i < array_size; i++)
				if ((pcode + i)->left[0] == (char)cin)
				{
					pcout = (pcode + i)->right;

					if (!bool_exclude || cin != sepchar)
						for (j = 0; j< strlen(pcout); j++)
							putc(*(pcout + j), fp_outfile);

					bool_wrote = TRUE;
					break;
				}

		if (!bool_wrote)
			putc(cin, fp_outfile);

		bool_wrote = FALSE;
	}


	debuglog((L_INF, func, "conversion finished"));

	fprintf(stderr, "status: %d%% complete...\n", 100);

	debuglog((L_DTL, func, "leaving"));

	return 0;
}

