static char rcs_id[] = "$Header: /home/kig/work/convert/src/RCS/cyrto.c,v 1.1 1996/10/20 19:37:31 kig Exp kig $";
/*******************************************************************************
 *
 * FILENAME:	cyrto.c
 * MODULE:		convert
 * NOTES:		Cyrillic transliteration routines (TO)
 *				translit_to_cyr() conversion
 *
 * Copyright (c) 1996 Konstantin Gredeskoul
 * All rights reserved
 *
 * Internet:	kig@cyber.com.au
 * Post:		P.O.Box 500, Carnegie, VIC 3163, Australia
 *
 ******************************************************************************/


/* includes */
#include <stdio.h>
#include <sys/stat.h>

#include "debuglog.h"
#include "bool.h"
#include "main.h"
#include "cyrto.h"

int translit_to_cyr(
	FILE *fp_infile, 			/* input file pointer */ 
	FILE *fp_outfile, 			/* output file pointer */
	char sepchar				/* exclude character */
	)
{
/*******************************************************************************
 *
 * PURPOSE
 *	This function reads text (written supposingly in Russian using English
 *	characters) and converts it to Russian text in Alternative encoding
 *	for MS-DOS using widely accepted transliteration standards. This function
 *	does a good job, but will not perform 100% correctly. The text will need
 *	to be edited after conversion to fix omissions and conversion errors.
 *
 ******************************************************************************/

	int			cin = 0;
	char		cin_prev = 0;
	char		cout = 0;

	/* boolean variables */
	int			bool_exclude = FALSE;
	int			bool_convert = TRUE;
	int			mode = MODE_OK;

	/* fast looping variables */
	long		cnt = 0;
	long		infile_size = -1;

	char		func[] = "translit_to_cyr";

	int			perc_curr = -1;
	int			perc_prev = -1;

	int			fd;
	struct stat	stat_buff;


	debuglog((L_DTL, func, "entering"));


	/* get source file size */

	fd = fileno(fp_infile);
	if (fstat(fd, &stat_buff) != 0)
		debuglog((L_ERR, func, "couldn't get file stat for input file"));
	else
	{
		infile_size = stat_buff.st_size;
		debuglog((L_INF, func, "got input file size = %dKb",
			infile_size / KILOBYTE ));
	}

	/* check if exclusion character is being used */
	if (sepchar != '\0')
		bool_exclude = TRUE;

	debuglog((L_DTL, func, "starting conversion"));

	/* start main loop, initialize mode */
	mode = MODE_OK;

	/*
	 * the trick here is that if a mode is MODE_SKIP_READ then
	 * in ANSI C the getc() part will not be processed, since
	 * the first bit of OR statement is already TRUE. We rely on that 
	 * because when MODE_SKIP_READ is TRUE we do NOT wish to read another
	 * character from file, rather we wish to process the previous character
	 * as if it was read from file. Since only when a new character is read
	 * do we increment the byte counter cnt, it is joined with getc() using 
	 * && into the statement with getc().
	 */
	while (mode == MODE_SKIP_READ || 
		(((cin = getc(fp_infile)) != EOF) && ++cnt))
	{

		/* update status display */
		if (cnt % PERCENT_BLOCK == 0)
		{
			perc_curr = (int)(100.0 * (float)cnt/(float)infile_size);
			if (perc_curr > perc_prev)
			{
				fprintf(stderr, "status: %d%% complete...\r", perc_curr);
				perc_prev = perc_curr;
			}
		}

		/* if exclude character was specified... */
		if (bool_exclude)
		{
			if (cin == sepchar)
			{
				bool_convert = !bool_convert;
				debuglog((L_INF, func, 
					"found on/off char <%c> at byte <%d>", sepchar, cnt));
				if (bool_convert)
					debuglog((L_INF, func, "converting is ON"));
				else
					debuglog((L_INF, func, "converting is OFF"));

				/* if mode is MODE_WAIT then there is a character
				 * pending to be processed. Since the current cin is
				 * sepchar, we just wish to print the previous cout
				 * as if it was printed last time. sepchar itself is not
				 * printed in any case.
				 */
				if (mode == MODE_WAIT)
					putc(cout, fp_outfile);
			
				/* we do not want to proceed any further in this case */
				continue;
			}

			/* 
			 * if we are excluding characters AND not converting stuff,
			 * then just assign cout to cin and continue.
			 */
			if (bool_convert == FALSE)
			{
				cout = cin;
				putc(cout, fp_outfile);
				continue;
			}
		}
		

		/* 
		 * here is where the serious logical stuff is done. 
		 * Brief idea: if we read character like, say, "a" then we
		 * just convert it using one-to-one relations below and that
		 * is done. This refers to MODE_OK. If we read something like
		 * "s" - this means we must look at the next character that follows
		 * "s"... This corresponds to MODE_WAIT. We read the next character
		 * and if it's, say, "h" then both are converted into just one russian
		 * letter "sh". If it's, say, "o" then both "s" and "o" have their 
		 * own related conversion. So we putc() the cout that was stored 
		 * when "s" was read and then set mode to MODE_SKIP_READ to process 
		 * the second character - "o" in our case. That's how it works.
		 */

		if (mode == MODE_OK || mode == MODE_SKIP_READ) 
		{
			switch	(cin) 
			{
				/* one-to-one conversions */
				case '\'': cout='\xec'; break;
				case 'a': cout='\xa0'; break;	case 'A': cout='\x80'; break;
				case 'b': cout='\xa1'; break;	case 'B': cout='\x81'; break;
				case 'v': cout='\xa2'; break;	case 'V': cout='\x82'; break;
				case 'g': cout='\xa3'; break;	case 'G': cout='\x83'; break;
				case 'd': cout='\xa4'; break;	case 'D': cout='\x84'; break;
				case 'w': cout='\xa6'; break;	case 'W': cout='\x86'; break;
				case 'i': cout='\xa8'; break;	case 'I': cout='\x88'; break;
				case 'k': cout='\xaa'; break;	case 'K': cout='\x8a'; break;
				case 'l': cout='\xab'; break;	case 'L': cout='\x8b'; break;
				case 'm': cout='\xac'; break;	case 'M': cout='\x8c'; break;
				case 'n': cout='\xad'; break;	case 'N': cout='\x8d'; break;
				case 'o': cout='\xae'; break;	case 'O': cout='\x8e'; break;
				case 'p': cout='\xaf'; break;	case 'P': cout='\x8f'; break;
				case 'r': cout='\xe0'; break;	case 'R': cout='\x90'; break;
				case 't': cout='\xe2'; break;	case 'T': cout='\x92'; break;
				case 'u': cout='\xe3'; break;	case 'U': cout='\x93'; break;
				case 'f': cout='\xe4'; break;	case 'F': cout='\x94'; break;
				case 'h': cout='\xe5'; break;	case 'H': cout='\x95'; break; 

				case 'c': cout='\xe6'; cin_prev = cin; mode = MODE_WAIT; break;
				case 'C': cout='\x96'; cin_prev = cin; mode = MODE_WAIT; break;
				case 'y': cout='\xa9'; cin_prev = cin; mode = MODE_WAIT; break;
				case 'Y': cout='\x89'; cin_prev = cin; mode = MODE_WAIT; break;
				case 'j': cout='\xa9'; cin_prev = cin; mode = MODE_WAIT; break;
				case 'J': cout='\x89'; cin_prev = cin; mode = MODE_WAIT; break;
				case 's': cout='\xe1'; cin_prev = cin; mode = MODE_WAIT; break;
				case 'S': cout='\x91'; cin_prev = cin; mode = MODE_WAIT; break;
				case 'z': cout='\xa7'; cin_prev = cin; mode = MODE_WAIT; break;
				case 'Z': cout='\x87'; cin_prev = cin; mode = MODE_WAIT; break;
		
				case 'e': cout='\xa5'; break;	case 'E': cout='\x85'; break;
	default : cout = cin;
			}
			if (mode == MODE_SKIP_READ)
				mode = MODE_OK;
		}
		else if (mode == MODE_WAIT)
		{
			/* two-to-one conversions */

			mode = MODE_OK;

		   	     if (cin_prev == 's' && cin == 'h') cout = '\xe8';
			else if (cin_prev == 'c' && cin == 'h') cout = '\xe7';
			else if (cin_prev == 'z' && cin == 'h') cout = '\xa6';
			else if (cin_prev == 'y' && cin == 'u') cout = '\xee';
			else if (cin_prev == 'y' && cin == 'a') cout = '\xef';
			else if (cin_prev == 'y' && cin == 'o') cout = '\xf1';
			else if (cin_prev == 'j' && cin == 'u') cout = '\xee';
			else if (cin_prev == 'j' && cin == 'a') cout = '\xef';
			else if (cin_prev == 'j' && cin == 'o') cout = '\xf1';

			else if (cin_prev == 'S' && cin == 'h') cout = '\x98';
			else if (cin_prev == 'C' && cin == 'h') cout = '\x97';
			else if (cin_prev == 'Z' && cin == 'h') cout = '\x86';
			else if (cin_prev == 'Y' && cin == 'u') cout = '\x9e';
			else if (cin_prev == 'Y' && cin == 'a') cout = '\x9f';
			else if (cin_prev == 'Y' && cin == 'o') cout = '\xf0';
			else if (cin_prev == 'J' && cin == 'u') cout = '\x9e';
			else if (cin_prev == 'J' && cin == 'a') cout = '\x9f';
			else if (cin_prev == 'J' && cin == 'o') cout = '\xf0';

			else if (cin_prev == 'S' && cin == 'H') cout = '\x98';
			else if (cin_prev == 'C' && cin == 'H') cout = '\x97';
			else if (cin_prev == 'Z' && cin == 'H') cout = '\x86';
			else if (cin_prev == 'Y' && cin == 'U') cout = '\x9e';
			else if (cin_prev == 'Y' && cin == 'A') cout = '\x9f';
			else if (cin_prev == 'Y' && cin == 'O') cout = '\xf0';
			else if (cin_prev == 'J' && cin == 'U') cout = '\x9e';
			else if (cin_prev == 'J' && cin == 'A') cout = '\x9f';
			else if (cin_prev == 'J' && cin == 'O') cout = '\xf0';
			
			else
				mode = MODE_SKIP_READ;
		}

		if (mode != MODE_WAIT)
			putc(cout, fp_outfile);
	}

	debuglog((L_INF, func, "conversion finished"));

	fprintf(stderr, "status: %d%% complete...\n", 100);

	debuglog((L_DTL, func, "leaving"));

	return 0;
}

