static char rcs_id[] = "$Header: /home/kig/work/convert/src/RCS/main.c,v 1.1 1996/10/20 19:37:31 kig Exp kig $";
/*******************************************************************************
 *
 * FILENAME:	main.c
 * MODULE:		convert
 * NOTES:		set tabsize to 4
 *
 * Copyright (c) 1996 Konstantin Gredeskoul
 * All rights reserved
 *
 * Internet:	kig@cyber.com.au
 * Post:		P.O.Box 500, Carnegie, VIC 3163, Australia
 *
 ******************************************************************************/

/* includes */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <time.h>

/* local includes */
#include "bool.h"
#include "debuglog.h"
#include "main.h"

#include "getopt.h"


int main(int argc, char *argv[])
{
/*******************************************************************************
 *
 * DESCRIPTION
 *	This is the main() function with central file handling, options resolving
 *	and looping.
 *
 ******************************************************************************/

	int					c;

	time_t				time_init;
	double				time_diff;

	extern char			*optarg;
	extern int			optind;

	int					opt_err = 0;
	int					opt_log = 0;
	int					opt_codefile = 0;
	int					opt_outfile = 0;
	int					opt_translit = 0;

	char				func[] = "main";
	char				scmd[KILOBYTE];
	char				stmp[KILOBYTE];
	int					i = 0;
	char				*env = NULL;


	CONFIG_DATA			cfg;
	CODE_DATA			codes;

    time_init = time(NULL);

	/* print header */
	fprintf(stderr, "%s", prginfo);

	/* initialize config structure */
	memset(&cfg, '\0', sizeof(cfg));

	cfg.reverse_mode = FALSE;
	cfg.sepchar = '\0';

	/* go through options and check usage */

	/* use normal getopt() if compiling under UNIX; or use
	 * supplied gnu_getopt() from getopt.c file if this is
	 * not UNIX
	 */
	while ((c = gnu_getopt(argc, argv, "th?l:c:ro:x:")) != EOF)
	{
		switch (c)
		{
			case 'l': /* log file - open and store filename */

				strcpy(cfg.logfile, optarg);
				if (debuglog_open(cfg.logfile) != 0)
				{
					fprintf(stderr, 
					"error: cannot open debuglog file <%s>.\n", cfg.logfile);
					exit(10);
				}

				opt_log++;
				break;

			case 'h':
			case '?':
				usage();
				exit(0);
				break;

			case 't':
				opt_translit++;
				break;

			case 'c':
				strcpy(cfg.codefile, optarg);
				opt_codefile++;
				break;

			case 'r':
				cfg.reverse_mode = TRUE;
				break;

			case 'o':
				strcpy(cfg.outfile, optarg);
				opt_outfile++;
				break;

			case 'x':
				cfg.sepchar = optarg[0];
				break;

		}
	}

	/*
	 * check if log file was opened
	 */
	if (opt_log > 0)
	{
		debuglog_level_set(6);
	}
	else
	{
		debuglog_level_set(-1);
	}
		
	debuglog((L_INF, func, "entering"));

	memset(scmd, '\0', sizeof(scmd));
	for (i = 0; i < argc; i++)
	{
		strcat(scmd, argv[i]);
		if (i != argc - 1)
			strcat(scmd, " ");
	}
	debuglog((L_INF, func, "command line: <%s>", scmd));

	/* 
	 * test for errors
	 */
	if (opt_err > 0)
	{
		debuglog((L_ERR, func, "incorrect usage"));
		fprintf(stderr, "error: incorrect usage; run convert -h to fix.\n");
		exit(20);
	}

	if (opt_translit > 0 && opt_codefile > 0)
	{
		debuglog((L_ERR, func, "incorrect usage - cannot use -t and -c"));
		fprintf(stderr, "error: incorrect usage - cannot use -t and -c.\n");
		exit(30);
	}

	if (opt_translit + opt_codefile == 0)
	{
		debuglog((L_FTL, func,
			"neither code file nor translit mode specified"));
		fprintf(stderr,
			"error: specify at least one of -t, -c or -h/-?.\n\n");
		debuglog_close();
		exit(40);
	}

	debuglog((L_INF, func, "starting after command line parameters parsed"));


	/*
	 * Test for sepchar
	 */
	if (cfg.sepchar == '\0')
		debuglog((L_INF, func, "not using exclude-character"));
	else
		debuglog((L_INF, func, "using exclude-character '%c'",
			cfg.sepchar));


	/*
	 * test for reverse mode
	 */
	if (cfg.reverse_mode == TRUE)
		debuglog((L_INF, func, "using REVERSE conversion mode"));
	else
		debuglog((L_INF, func, "using STRAIGHT conversion mode"));


	/*
	 * determine and open input file
	 */
	if (optind > argc - 1)
	{
		strcpy(cfg.infile, "");
		cfg.fp_infile = stdin;
		debuglog((L_INF, func, "reading data from <stdin>"));
	}
	else
	{	
		strcpy(cfg.infile, argv[optind]);
		debuglog((L_INF, func, "reading data from <%s>", cfg.infile));
		cfg.fp_infile = fopen(cfg.infile, "r");

		if (cfg.fp_infile == NULL)
		{
			debuglog((L_FTL, func, "error opening file <%s> for reading", 
				cfg.infile));
			fprintf(stderr, "error: opening file <%s> for reading\n",
				cfg.infile);
			debuglog_close();
			exit(50);
		}
		else
			debuglog((L_INF, func, "file <%s> opened for reading",
				cfg.infile));
	}


	/*
	 * Determine and open output file
	 */
	if (opt_outfile == 0)
	{
		strcpy(cfg.outfile, "");
		cfg.fp_outfile = stdout;
		debuglog((L_INF, func, "writing output to <stdout>"));
	}
	else
	{
		debuglog((L_INF, func, "writing output to <%s>", cfg.outfile));
		cfg.fp_outfile = fopen(cfg.outfile, "w");
		if (cfg.fp_outfile == NULL)
		{
			debuglog((L_FTL, func, "error opening file <%s> for writing", 
				cfg.outfile));
			fprintf(stderr, "error: opening file <%s> for writing\n",
				cfg.outfile);
			debuglog_close();
			exit(60);
		}
		else
			debuglog((L_INF, func, "file <%s> opened for writing",
				cfg.outfile));
	}


/******************************************************************************/

	if (opt_translit > 0)
	{
		debuglog((L_INF, func, "transliteration mode selected"));

		if (cfg.reverse_mode == TRUE)
			translit_from_cyr(
				cfg.fp_infile, 
				cfg.fp_outfile, 
				cfg.sepchar
				);
		else
			translit_to_cyr(
				cfg.fp_infile, 
				cfg.fp_outfile,
				cfg.sepchar
				);
	} 

/******************************************************************************/

	else if (opt_codefile > 0)
	{
		debuglog((L_INF, func,
			"trying to open code file <./%s>", cfg.codefile));

		if ((cfg.fp_codefile = fopen(cfg.codefile, "r")) == NULL)
		{
			debuglog((L_WRN, func,
				"couldn't open code file <./%s>", cfg.codefile));

			/* 
			 * get environment variable; if unset
			 * use defaults
			 */
			if ((env=(char *)getenv(ENV_CODE_FILE_PATH)) == NULL)
			{
				debuglog((L_INF, func,
					"environment %s not set... using defaults",
					ENV_CODE_FILE_PATH
					));

				env = DFL_CODE_FILE_PATH;
			}
			else
				debuglog((L_INF, func,
					"environment %s is set to <%s>",
					ENV_CODE_FILE_PATH,
					env
					));
			
			/* concatenate value of env and cfg.codefile together */
			sprintf(stmp, "%s/%s", env, cfg.codefile);
			debuglog((L_INF, func, 
				"trying to open code file <%s>", 
				stmp
				));
			

			if ((cfg.fp_codefile = fopen(stmp, "r")) == NULL)
			{
				debuglog((L_WRN, func,
					"couldn't open code file <%s>",
					stmp
					));

				sprintf(stmp, "%s/%s", DFL_CODE_FILE_PATH, cfg.codefile);

				debuglog((L_INF, func, 
					"finally trying to open code file <%s>", 
					stmp
					));

				if ((cfg.fp_codefile = fopen(stmp, "r")) == NULL)
				{
					debuglog((L_FTL, func, 
						"error opening code file <%s> for reading", 
						stmp
						));

					debuglog((L_FTL, func,
						"fatal: searched in <./>, default <%s>",
						DFL_CODE_FILE_PATH
						));

					debuglog((L_FTL, func,
						"and in the path pointed to by "
						"%s environment variable",
						ENV_CODE_FILE_PATH
						));

					debuglog((L_FTL, func,
						"code file was not found. Good bye."));

					fprintf( stderr,
						"error: cannot open file <%s> for reading\n",
						cfg.codefile
						);

					debuglog_close();
					exit(70);
				}
			}
		}

		debuglog((L_INF, func, 
			"code file opened for reading successfully",
			cfg.codefile));

		load_codes(
			cfg.fp_codefile, 
			&codes
			);

		convert(
			cfg.fp_infile, 
			cfg.fp_outfile, 
			&codes, 
			cfg.reverse_mode, 
			cfg.sepchar
			);

		if (fclose(cfg.fp_codefile) == 0)
			debuglog((L_INF, func, "code file <%s> closed",
				cfg.codefile));
		else
			debuglog((L_ERR, func, "there was an error closing code file <%s>",
				cfg.codefile));
	}


/******************************************************************************/

	if (cfg.fp_outfile != stdout) 
	{
		if (fclose(cfg.fp_outfile) == 0)
			debuglog((L_INF, func, "output file <%s> closed",
				cfg.outfile));
		else
			debuglog((L_ERR, func, "there was an error closing file <%s>",
				cfg.outfile));
	}


	if (cfg.fp_infile != stdin) 
	{
		if (fclose(cfg.fp_infile) == 0)
			debuglog((L_INF, func, "input file <%s> closed",
				cfg.infile));
		else
			debuglog((L_ERR, func, "there was an error closing file <%s>",
				cfg.infile));
	}

/******************************************************************************/

    time_diff = difftime(time(NULL), time_init);
    fprintf(stderr, "total used %d seconds\n", (int)time_diff);

	debuglog((L_INF, func, "total %d seconds of runtime", (int)time_diff));
	debuglog((L_INF, func, "finished\n"));
	debuglog_close();

	fprintf(stderr, "all done. Good bye.\n");

	return (0);

} /* end main */





void usage(void)
{
/*******************************************************************************
 *
 * DESCRIPTION
 *	This function prints out the usage unformation.
 *
 ******************************************************************************/

	char	s[] = ""
	"usage:\tconvert -c file [-r] [-x char] [-l file] [-o file] [file]\n"
	"\tconvert -t [-r] [-l file] [-o file] [file]\n"
	"\tconvert -h | -?\n\n"
	"description:\n"
	"\tConvert transforms ASCII text files from one 'encoding system' to\n" 
	"\tanother. In '-t' mode it also transliterates Russian texts.\n"
	"\tflags:\n"
	"\t-h/-?\t\tinvoke this help screen (you may need to use -\\?)\n"
	"\t-c file\t\tspecifies a file with conversion codes (codes mode)\n"
	"\t\t\tSet environment variable " ENV_CODE_FILE_PATH " if code\n"
	"\t\t\tfiles are not in the current directory.\n"
	"\t-t\t\tcall Cyrillic transliteration (translit mode)\n"
	"\t-r\t\tuse reverse conversion or transliteration\n"
	"\t-x char\t\tenable on/off character to exclude some parts of text\n"
	"\t-o file\t\toutput file. If not given stdout is used.\n"
	"\t-l file\t\tspecifies a file for logging debug messages\n"
	"\tfile\t\tfile to be converted. If not given stdin is used.\n"
	;

	fprintf(stderr, "%s", s);
}

